package org.apache.bcel.classfile;

import java.util.Arrays;
import java.io.DataOutputStream;
import org.apache.commons.lang3.ArrayUtils;
import java.io.IOException;
import java.io.DataInput;

public final class NestMembers extends Attribute
{
    private int[] classes;
    
    NestMembers(final int nameIndex, final int length, final DataInput input, final ConstantPool constantPool) throws IOException {
        this(nameIndex, length, (int[])null, constantPool);
        final int classCount = input.readUnsignedShort();
        this.classes = new int[classCount];
        for (int i = 0; i < classCount; ++i) {
            this.classes[i] = input.readUnsignedShort();
        }
    }
    
    public NestMembers(final int nameIndex, final int length, final int[] classes, final ConstantPool constantPool) {
        super((byte)26, nameIndex, length, constantPool);
        this.classes = ((classes != null) ? classes : ArrayUtils.EMPTY_INT_ARRAY);
    }
    
    public NestMembers(final NestMembers c) {
        this(c.getNameIndex(), c.getLength(), c.getClasses(), c.getConstantPool());
    }
    
    @Override
    public void accept(final Visitor v) {
        v.visitNestMembers(this);
    }
    
    @Override
    public Attribute copy(final ConstantPool constantPool) {
        final NestMembers c = (NestMembers)this.clone();
        if (this.classes != null) {
            c.classes = this.classes.clone();
        }
        c.setConstantPool(constantPool);
        return c;
    }
    
    @Override
    public void dump(final DataOutputStream file) throws IOException {
        super.dump(file);
        file.writeShort(this.classes.length);
        for (final int index : this.classes) {
            file.writeShort(index);
        }
    }
    
    public int[] getClasses() {
        return this.classes;
    }
    
    public String[] getClassNames() {
        final String[] names = new String[this.classes.length];
        Arrays.setAll(names, i -> Utility.pathToPackage(super.getConstantPool().getConstantString(this.classes[i], (byte)7)));
        return names;
    }
    
    public int getNumberClasses() {
        return (this.classes == null) ? 0 : this.classes.length;
    }
    
    public void setClasses(final int[] classes) {
        this.classes = ((classes != null) ? classes : ArrayUtils.EMPTY_INT_ARRAY);
    }
    
    @Override
    public String toString() {
        final StringBuilder buf = new StringBuilder();
        buf.append("NestMembers(");
        buf.append(this.classes.length);
        buf.append("):\n");
        for (final int index : this.classes) {
            final String className = super.getConstantPool().getConstantString(index, (byte)7);
            buf.append("  ").append(Utility.compactClassName(className, false)).append("\n");
        }
        return buf.substring();
    }
}
